@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtWidgets module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -134,7 +140,7 @@ public:
         PE_FrameGroupBox,
         PE_FrameLineEdit,
         PE_FrameMenu,
-        PE_FrameStatusBar, // obsolete
+        PE_FrameStatusBar, // ### Qt 6: remove
         PE_FrameStatusBarItem = PE_FrameStatusBar,
         PE_FrameTabWidget,
         PE_FrameWindow,
@@ -155,7 +161,7 @@ public:
         PE_IndicatorArrowUp,
         PE_IndicatorBranch,
         PE_IndicatorButtonDropDown,
-        PE_IndicatorViewItemCheck,
+        PE_IndicatorViewItemCheck, // ### Qt 6: remove
         PE_IndicatorItemViewItemCheck = PE_IndicatorViewItemCheck,
         PE_IndicatorCheckBox,
         PE_IndicatorDockWidgetResizeHandle,
@@ -171,6 +177,7 @@ public:
         PE_IndicatorToolBarSeparator,
         PE_PanelTipLabel,
         PE_IndicatorTabTear,
+        PE_IndicatorTabTearLeft = PE_IndicatorTabTear,
         PE_PanelScrollAreaCorner,
 
         PE_Widget,
@@ -186,6 +193,8 @@ public:
         PE_IndicatorTabClose,
         PE_PanelMenu,
 
+        PE_IndicatorTabTearRight,
+
         // do not add any values below/greater this
         PE_CustomBase = 0xf000000
     };
@@ -298,10 +307,11 @@ public:
         SE_TabWidgetLeftCorner,
         SE_TabWidgetRightCorner,
 
-        SE_ViewItemCheckIndicator,
+        SE_ViewItemCheckIndicator, // ### Qt 6: remove
         SE_ItemViewItemCheckIndicator = SE_ViewItemCheckIndicator,
 
         SE_TabBarTearIndicator,
+        SE_TabBarTearIndicatorLeft = SE_TabBarTearIndicator,
 
         SE_TreeViewDisclosureItem,
 
@@ -341,6 +351,10 @@ public:
 
         SE_ToolBarHandle,
 
+        SE_TabBarScrollLeftButton,
+        SE_TabBarScrollRightButton,
+        SE_TabBarTearIndicatorRight,
+
         // do not add any values below/greater than this
         SE_CustomBase = 0xf0000000
     };
@@ -491,9 +505,9 @@ public:
         PM_DialogButtonsButtonHeight,
 
         PM_MdiSubWindowFrameWidth,
-        PM_MDIFrameWidth = PM_MdiSubWindowFrameWidth,            //obsolete
+        PM_MDIFrameWidth = PM_MdiSubWindowFrameWidth,         // ### Qt 6: remove
         PM_MdiSubWindowMinimizedWidth,
-        PM_MDIMinimizedWidth = PM_MdiSubWindowMinimizedWidth,    //obsolete
+        PM_MDIMinimizedWidth = PM_MdiSubWindowMinimizedWidth, // ### Qt 6: remove
 
         PM_HeaderMargin,
         PM_HeaderMarkSize,
@@ -511,9 +525,9 @@ public:
 
         PM_SpinBoxSliderHeight,
 
-        PM_DefaultTopLevelMargin,
-        PM_DefaultChildMargin,
-        PM_DefaultLayoutSpacing,
+        PM_DefaultTopLevelMargin, // ### Qt 6: remove
+        PM_DefaultChildMargin,    // ### Qt 6: remove
+        PM_DefaultLayoutSpacing,  // ### Qt 6: remove
 
         PM_ToolBarIconSize,
         PM_ListViewIconSize,
@@ -631,7 +645,7 @@ public:
         SH_ComboBox_Popup,
         SH_TitleBar_NoBorder,
         SH_Slider_StopMouseOverSlider,
-        SH_ScrollBar_StopMouseOverSlider = SH_Slider_StopMouseOverSlider, // obsolete
+        SH_ScrollBar_StopMouseOverSlider = SH_Slider_StopMouseOverSlider, // ### Qt 6: remove
         SH_BlinkCursorWhenTextSelected,
         SH_RichText_FullWidthSelection,
         SH_Menu_Scrollable,
@@ -718,6 +732,7 @@ public:
         SH_Menu_SubMenuSloppyCloseTimeout,
         SH_Menu_SubMenuResetWhenReenteringParent,
         SH_Menu_SubMenuDontStartSloppyOnLeave,
+        SH_ItemView_ScrollMode,
         // Add new style hint values here
 
         SH_CustomBase = 0xf0000000
